from .role_permission_model import RolePermissionModel
from .user_role_model import UserRoleModel

from .auth_code_model import AuthCodeModel
from .permission_model import PermissionModel
from .role_model import RoleModel
from .user_model import UserModel

__all__ = [
    "AuthCodeModel",
    "RoleModel",
    "PermissionModel",
    "RolePermissionModel",
    "UserModel",
    "UserRoleModel",
]



from litestar.status_codes import HTTP_200_OK, HTTP_201_CREATED, HTTP_204_NO_CONTENT

from tests.factories import RoleFactory


async def test_get_roles(admin_api_client):
    response = await admin_api_client.get("/api/auth/roles")
    assert response.status_code == HTTP_200_OK


async def test_create_role(admin_api_client, roles_fixture, role_service):
    prev_items_count = await role_service.count()

    response = await admin_api_client.post("/api/auth/roles", json={"name": "superadmin"})
    assert response.status_code == HTTP_201_CREATED

    items_count = await role_service.count()
    assert items_count == prev_items_count + 1

    # teardown
    await role_service.delete(response.json()["id"])


async def test_remove_role(admin_api_client, roles_fixture, role_service):
    roles_count = await role_service.count()

    role_to_remove = await role_service.create(RoleFactory.build(), auto_commit=True)
    response = await admin_api_client.delete(f"/api/auth/roles/{role_to_remove.id}")
    assert response.status_code == HTTP_204_NO_CONTENT
    assert roles_count == await role_service.count()



from litestar.openapi import OpenAPIConfig
from litestar.openapi.plugins import ScalarRenderPlugin
from litestar.openapi.spec import Components, SecurityScheme

from .settings import settings

openapi_config = OpenAPIConfig(
    components=[
        Components(
            security_schemes={
                "JWT": SecurityScheme(
                    type="http",
                    scheme="Bearer",
                    name="Authorization",
                    security_scheme_in="cookie",
                    bearer_format="JWT",
                    description="Authorization",
                )
            }
        )
    ],
    title=settings.open_api_config.title,
    version=settings.open_api_config.version,
    path=settings.open_api_config.path,
    render_plugins=[ScalarRenderPlugin()],
)



"""update auth_code model unique

Revision ID: d4cdc9db5ed8
Revises: fafdb22e5a16
Create Date: 2026-01-23 15:45:07.166844

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
import advanced_alchemy


# revision identifiers, used by Alembic.
revision: str = 'd4cdc9db5ed8'
down_revision: Union[str, Sequence[str], None] = 'fafdb22e5a16'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(op.f('uq_auth_codes_user_id'), 'auth_codes', type_='unique')
    op.create_unique_constraint('unique_user_action', 'auth_codes', ['user_id', 'action'])
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint('unique_user_action', 'auth_codes', type_='unique')
    op.create_unique_constraint(op.f('uq_auth_codes_user_id'), 'auth_codes', ['user_id'], postgresql_nulls_not_distinct=False)
    # ### end Alembic commands ###



from advanced_alchemy.base import UUIDAuditBase
from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, mapped_column


class UserRoleModel(UUIDAuditBase):
    __tablename__ = "users_roles"

    role_id: Mapped[int] = mapped_column(ForeignKey("roles.id", ondelete="CASCADE"), primary_key=True)
    user_id: Mapped[int] = mapped_column(ForeignKey("users.id"), primary_key=True)



from advanced_alchemy.base import UUIDAuditBase
from advanced_alchemy.config.asyncio import AlembicAsyncConfig
from litestar.plugins.sqlalchemy import SQLAlchemyAsyncConfig, SQLAlchemyPlugin
from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker, create_async_engine

from .settings import settings

config = SQLAlchemyAsyncConfig(
    connection_string=settings.db_url,
    metadata=UUIDAuditBase.metadata,
    create_all=settings.db_create_all,
    alembic_config=AlembicAsyncConfig(
        script_location="./migrations/",
    ),
)
sqlalchemy_plugin = SQLAlchemyPlugin(config=config)


engine = create_async_engine(settings.db_url, echo=False)
session_maker = async_sessionmaker(engine, expire_on_commit=False, class_=AsyncSession)



from advanced_alchemy.repository import SQLAlchemyAsyncRepository
from advanced_alchemy.service import SQLAlchemyAsyncRepositoryService
from sqlalchemy.ext.asyncio import AsyncSession

from ..models import UserModel

__all__ = [
    "UserService",
    "provide_user_service",
]


class UserService(SQLAlchemyAsyncRepositoryService):
    class Repository(SQLAlchemyAsyncRepository[UserModel]):
        model_type = UserModel

    repository_type = Repository


async def provide_user_service(db_session: AsyncSession) -> UserService:
    return UserService(session=db_session)



from typing import Any
from uuid import UUID

from litestar import Controller, Request, get, post
from litestar.datastructures import State
from litestar.di import Provide
from litestar.exceptions import HTTPException
from litestar.status_codes import HTTP_201_CREATED, HTTP_400_BAD_REQUEST
import msgspec

from auth.services import (
    AccountDTO,
    AccountUserDTO,
    AuthService,
    ConfirmResetPasswordWrongCodeError,
    ConfirmResetPasswordRequestDTO,
    EmailIsAlreadyUsedError,
    InvalidEmailError,
    InvalidPasswordError,
    LoginRequestDTO,
    LogoutRequestDTO,
    SignUpRequestDTO,
    StartResetPasswordEmailNotFoundError,
    UserEmailNotVerifiedError,
    UserNotEnabledError,
    WrongAuthCodeError,
    provide_auth_service,
    provide_user_service,
)


class SignUpRequestScheme(msgspec.Struct):
    email: str
    password: str


class SignUpResponseScheme(msgspec.Struct):
    message: str


class StartResetPasswordResponseScheme(msgspec.Struct):
    message: str


class LoginResponseScheme(AccountDTO): ...


class ConfirmResetPasswordResponseScheme(AccountDTO): ...


class VerifyEmailResponseScheme(AccountDTO): ...


class LoginRequestScheme(msgspec.Struct):
    email: str
    password: str


class VerifyEmailRequestScheme(msgspec.Struct):
    code: str


class StartResetPasswordRequestScheme(msgspec.Struct):
    email: str


class ConfirmResetPasswordRequestScheme(msgspec.Struct):
    code: str
    password: str


class AccountMeResponseScheme(msgspec.Struct):
    session_id: UUID
    user: AccountUserDTO


class AccountController(Controller):
    path = "/account"

    dependencies = {
        "user_service": Provide(provide_user_service),
        "auth_service": Provide(provide_auth_service),
    }

    @get("/me")
    async def account(self, request: Request[AccountDTO, Any, State]) -> AccountMeResponseScheme:
        return AccountMeResponseScheme(session_id=request.user.session_id, user=request.user.user)

    @post("/login", exclude_from_auth=True)
    async def login(self, request: Request, data: LoginRequestScheme, auth_service: AuthService) -> LoginResponseScheme:
        device = request.headers.get("User-Agent", "Unknown")
        try:
            login_user = await auth_service.login(
                LoginRequestDTO(
                    email=data.email,
                    password=data.password,
                    device=device,
                )
            )
        except UserNotEnabledError:
            raise HTTPException("User is not enabled", status_code=HTTP_400_BAD_REQUEST)
        except UserEmailNotVerifiedError:
            raise HTTPException("Email not verified", status_code=HTTP_400_BAD_REQUEST)
        except InvalidEmailError:
            raise HTTPException("Email not found", status_code=HTTP_400_BAD_REQUEST)
        except InvalidPasswordError:
            raise HTTPException("Invalid password", status_code=HTTP_400_BAD_REQUEST)

        return LoginResponseScheme(
            token=login_user.token,
            session_id=login_user.session_id,
            user=login_user.user,
        )

    @post("/signup", exclude_from_auth=True)
    async def sign_up(self, data: SignUpRequestScheme, auth_service: AuthService) -> SignUpResponseScheme:
        try:
            await auth_service.signup(SignUpRequestDTO(email=data.email, password=data.password))
        except EmailIsAlreadyUsedError:
            raise HTTPException("Email is already used", status_code=HTTP_400_BAD_REQUEST)
        return SignUpResponseScheme(message="success")

    @post("/logout")
    async def logout(self, auth_service: AuthService, request: Request[AccountDTO, Any, State]) -> None:
        await auth_service.logout(
            LogoutRequestDTO(
                user_id=request.user.user.id,
                session_id=request.user.session_id,
                device=request.headers.get("User-Agent", "Unknown"),
            )
        )

    @post("/verify-email", exclude_from_auth=True)
    async def verify_email(
        self,
        data: VerifyEmailRequestScheme,
        auth_service: AuthService,
        request: Request,
    ) -> VerifyEmailResponseScheme:
        device = request.headers.get("User-Agent")

        try:
            activated_account = await auth_service.verify_user_email(data.code, device)
        except WrongAuthCodeError:
            raise HTTPException("Wrong verification code", status_code=HTTP_400_BAD_REQUEST)

        return VerifyEmailResponseScheme(
            token=activated_account.token,
            session_id=activated_account.session_id,
            user=activated_account.user,
        )

    @post("/start-reset-password", exclude_from_auth=True, status_code=HTTP_201_CREATED)
    async def start_reset_password(
        self,
        data: StartResetPasswordRequestScheme,
        auth_service: AuthService,
    ) -> StartResetPasswordResponseScheme:
        try:
            await auth_service.start_reset_password(data.email)
        except StartResetPasswordEmailNotFoundError:
            raise HTTPException("Start reset password failed, email not found", status_code=HTTP_400_BAD_REQUEST)

        return StartResetPasswordResponseScheme(message="success")

    @post("/confirm-reset-password", exclude_from_auth=True, status_code=HTTP_201_CREATED)
    async def confirm_reset_password(
        self,
        request: Request,
        data: ConfirmResetPasswordRequestScheme,
        auth_service: AuthService,
    ) -> ConfirmResetPasswordResponseScheme:
        device = request.headers.get("User-Agent", "Unknown")
        try:
            account = await auth_service.confirm_rest_password(
                ConfirmResetPasswordRequestDTO(
                    password=data.password,
                    device=device,
                    code=data.code,
                )
            )

            return ConfirmResetPasswordResponseScheme(
                token=account.token,
                session_id=account.session_id,
                user=account.user,
            )

        except ConfirmResetPasswordWrongCodeError:
            raise HTTPException("Confirm reset password failed, wrong code", status_code=HTTP_400_BAD_REQUEST)



from .auth_service import (
    API_KEY_HEADER,
    TOKEN_PREFIX,
    AccountDTO,
    AccountUserDTO,
    AuthCodeAction,
    AuthService,
    ConfirmResetPasswordRequestDTO,
    ConfirmResetPasswordWrongCodeError,
    DecodeTokenError,
    EmailIsAlreadyUsedError,
    InvalidEmailError,
    InvalidPasswordError,
    LoginRequestDTO,
    LogoutRequestDTO,
    SignUpRequestDTO,
    StartResetPasswordEmailNotFoundError,
    UserEmailNotVerifiedError,
    UserNotEnabledError,
    WrongAuthCodeError,
    provide_auth_service,
)
from .permission_service import PermissionService, provide_permission_service
from .role_service import RoleService, provide_role_service
from .user_service import UserService, provide_user_service

__all__ = [
    "UserService",
    "AuthService",
    "PermissionService",
    "RoleService",
    "AccountDTO",
    "AccountUserDTO",
    "ConfirmResetPasswordRequestDTO",
    "LogoutRequestDTO",
    "LoginRequestDTO",
    "SignUpRequestDTO",
    "AuthCodeAction",
    # providers
    "provide_user_service",
    "provide_auth_service",
    "provide_permission_service",
    "provide_role_service",
    # errors
    "InvalidPasswordError",
    "UserNotEnabledError",
    "UserEmailNotVerifiedError",
    "InvalidEmailError",
    "DecodeTokenError",
    "WrongAuthCodeError",
    "EmailIsAlreadyUsedError",
    "ConfirmResetPasswordWrongCodeError",
    "StartResetPasswordEmailNotFoundError",
    # constants
    "API_KEY_HEADER",
    "TOKEN_PREFIX",
]



from sre_parse import ASSERT
from typing import AsyncGenerator
from uuid import UUID

from litestar.exceptions import NotAuthorizedException
import msgspec
import pytest

from auth.services import (
    AccountDTO,
    AuthService,
    InvalidEmailError,
    InvalidPasswordError,
    LoginRequestDTO,
    LogoutRequestDTO,
    SignUpRequestDTO,
    UserEmailNotVerifiedError,
    UserNotEnabledError,
    UserService,
    WrongAuthCodeError,
)
from auth.services.auth_service import (
    ConfirmResetPasswordRequestDTO,
    ConfirmResetPasswordWrongCodeError,
    StartResetPasswordEmailNotFoundError,
)
from tests.factories import UserFactory
from tests.integrations import TestMailClient


class AuthInfoFixture(msgspec.Struct):
    user_id: UUID
    email: str
    code: str
    password: str


@pytest.fixture
async def auth_info(auth_service: AuthService) -> AsyncGenerator[AuthInfoFixture]:
    user = UserFactory.build()
    auth_code = await auth_service.signup(SignUpRequestDTO(email=user.email, password=user.password))
    yield AuthInfoFixture(
        user_id=auth_code.user_id,
        email=user.email,
        password=user.password,
        code=auth_code.code,
    )
    await auth_service.user_service.delete(auth_code.user_id)


async def test_auth_signup(auth_service: AuthService, mail_client: TestMailClient, user_service: UserService):
    user = UserFactory.build()
    auth_code = await auth_service.signup(SignUpRequestDTO(email=user.email, password=user.password))
    assert mail_client.messages == [([user.email], "Sign up", auth_code.code)]

    created_user = await user_service.get(auth_code.user_id)
    assert created_user.email == user.email

    # teardown
    await auth_service.user_service.delete(auth_code.user_id)


async def test_reset_password_success(auth_service: AuthService, auth_info: AuthInfoFixture):
    new_password = "new_password"

    auth_code = await auth_service.start_reset_password(auth_info.email)
    account_session = await auth_service.confirm_rest_password(
        ConfirmResetPasswordRequestDTO(code=auth_code.code, device="testclient", password=new_password)
    )

    assert account_session.user.id == auth_info.user_id

    login_account_session = await auth_service.login(
        LoginRequestDTO(email=auth_info.email, password=new_password, device="testclient")
    )

    assert login_account_session.user.id == auth_info.user_id


async def test_start_reset_password_multiple_times(auth_service: AuthService, auth_info: AuthInfoFixture):
    prev_auth_code = (await auth_service.start_reset_password(auth_info.email)).code
    next_auth_code = (await auth_service.start_reset_password(auth_info.email)).code

    assert prev_auth_code != next_auth_code


async def test_start_reset_password_wrong_email(auth_service: AuthService, auth_info: AuthInfoFixture):
    with pytest.raises(StartResetPasswordEmailNotFoundError):
        await auth_service.start_reset_password("wrong@mail.com")


async def test_confirm_reset_password_wrong_code(auth_service: AuthService, auth_info: AuthInfoFixture):
    new_password = "new_password"
    await auth_service.start_reset_password(auth_info.email)

    with pytest.raises(ConfirmResetPasswordWrongCodeError):
        await auth_service.confirm_rest_password(
            ConfirmResetPasswordRequestDTO(code="00000000", device="testclient", password=new_password)
        )


async def test_auth_login_success(auth_service: AuthService, auth_info: AuthInfoFixture):
    activated_user = await auth_service.verify_user_email(auth_info.code, "testclient")
    account_session = await auth_service.login(
        LoginRequestDTO(
            email=auth_info.email,
            password=auth_info.password,
            device="testclient",
        )
    )
    assert account_session.user.id == activated_user.user.id


async def test_auth_login_wrong_auth_code(auth_info: AuthInfoFixture, auth_service: AuthService):
    with pytest.raises(WrongAuthCodeError):
        await auth_service.verify_user_email("12345678", "testclient")


async def test_auth_login_user_email_not_verified(auth_info: AuthInfoFixture, auth_service: AuthService):
    with pytest.raises(UserEmailNotVerifiedError):
        await auth_service.login(
            LoginRequestDTO(email=auth_info.email, password=auth_info.password, device="testclient")
        )


async def test_auth_login_wrong_email(auth_service: AuthService, auth_info: AuthInfoFixture):
    await auth_service.verify_user_email(auth_info.code, "testclient")

    with pytest.raises(InvalidEmailError):
        await auth_service.login(
            LoginRequestDTO(email="wrong@mail.com", password=auth_info.password, device="testclient")
        )


async def test_auth_login_wrong_password(auth_service: AuthService, auth_info: AuthInfoFixture):
    await auth_service.verify_user_email(auth_info.code, "testclient")

    with pytest.raises(InvalidPasswordError):
        await auth_service.login(LoginRequestDTO(email=auth_info.email, password="wrong_password", device="testclient"))


async def test_auth_get_account(auth_service: AuthService, auth_info: AuthInfoFixture):
    activated_user = await auth_service.verify_user_email(auth_info.code, "testclient")
    account_session = await auth_service.login(
        LoginRequestDTO(
            email=auth_info.email,
            password=auth_info.password,
            device="testclient",
        )
    )
    assert account_session.user.id == activated_user.user.id
    account = await auth_service.get_account(account_session.token)
    assert account.user.id == activated_user.user.id


async def test_auth_logout_success(auth_service: AuthService, auth_info: AuthInfoFixture):
    activated_user = await auth_service.verify_user_email(auth_info.code, "testclient")
    account_session = await auth_service.login(
        LoginRequestDTO(
            email=auth_info.email,
            password=auth_info.password,
            device="testclient",
        )
    )
    account = await auth_service.get_account(account_session.token)

    await auth_service.logout(
        LogoutRequestDTO(
            user_id=activated_user.user.id,
            device="testclient",
            session_id=account_session.session_id,
        )
    )

    with pytest.raises(NotAuthorizedException):
        await auth_service.check_session(
            account,
            "testclient",
        )



"""init app

Revision ID: 491aaa1a58f6
Revises: 
Create Date: 2026-01-17 21:29:43.495198

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
import advanced_alchemy


# revision identifiers, used by Alembic.
revision: str = '491aaa1a58f6'
down_revision: Union[str, Sequence[str], None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('permissions',
    sa.Column('name', sa.String(), nullable=False),
    sa.Column('id', advanced_alchemy.types.guid.GUID(length=16), nullable=False),
    sa.Column('sa_orm_sentinel', sa.Integer(), nullable=True),
    sa.Column('created_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.Column('updated_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_permissions')),
    sa.UniqueConstraint('name', name=op.f('uq_permissions_name'))
    )
    op.create_table('roles',
    sa.Column('name', sa.String(), nullable=False),
    sa.Column('id', advanced_alchemy.types.guid.GUID(length=16), nullable=False),
    sa.Column('sa_orm_sentinel', sa.Integer(), nullable=True),
    sa.Column('created_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.Column('updated_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_roles')),
    sa.UniqueConstraint('name', name=op.f('uq_roles_name'))
    )
    op.create_table('users',
    sa.Column('email', sa.String(), nullable=False),
    sa.Column('password', sa.String(), nullable=False),
    sa.Column('is_email_verified', sa.Boolean(), nullable=False),
    sa.Column('is_enabled', sa.Boolean(), nullable=False),
    sa.Column('id', advanced_alchemy.types.guid.GUID(length=16), nullable=False),
    sa.Column('sa_orm_sentinel', sa.Integer(), nullable=True),
    sa.Column('created_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.Column('updated_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_users')),
    sa.UniqueConstraint('email', name=op.f('uq_users_email'))
    )
    op.create_table('auth_codes',
    sa.Column('code', sa.String(), nullable=False),
    sa.Column('user_id', advanced_alchemy.types.guid.GUID(length=16), nullable=True),
    sa.Column('id', advanced_alchemy.types.guid.GUID(length=16), nullable=False),
    sa.Column('sa_orm_sentinel', sa.Integer(), nullable=True),
    sa.Column('created_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.Column('updated_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.ForeignKeyConstraint(['user_id'], ['users.id'], name=op.f('fk_auth_codes_user_id_users'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_auth_codes')),
    sa.UniqueConstraint('user_id', name=op.f('uq_auth_codes_user_id'))
    )
    op.create_table('roles_permissions',
    sa.Column('role_id', advanced_alchemy.types.guid.GUID(length=16), nullable=False),
    sa.Column('permission_id', advanced_alchemy.types.guid.GUID(length=16), nullable=False),
    sa.Column('id', advanced_alchemy.types.guid.GUID(length=16), nullable=False),
    sa.Column('sa_orm_sentinel', sa.Integer(), nullable=True),
    sa.Column('created_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.Column('updated_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.ForeignKeyConstraint(['permission_id'], ['permissions.id'], name=op.f('fk_roles_permissions_permission_id_permissions'), ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['role_id'], ['roles.id'], name=op.f('fk_roles_permissions_role_id_roles')),
    sa.PrimaryKeyConstraint('role_id', 'permission_id', 'id', name=op.f('pk_roles_permissions'))
    )
    op.create_table('users_roles',
    sa.Column('role_id', advanced_alchemy.types.guid.GUID(length=16), nullable=False),
    sa.Column('user_id', advanced_alchemy.types.guid.GUID(length=16), nullable=False),
    sa.Column('id', advanced_alchemy.types.guid.GUID(length=16), nullable=False),
    sa.Column('sa_orm_sentinel', sa.Integer(), nullable=True),
    sa.Column('created_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.Column('updated_at', advanced_alchemy.types.datetime.DateTimeUTC(timezone=True), nullable=False),
    sa.ForeignKeyConstraint(['role_id'], ['roles.id'], name=op.f('fk_users_roles_role_id_roles'), ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['user_id'], ['users.id'], name=op.f('fk_users_roles_user_id_users')),
    sa.PrimaryKeyConstraint('role_id', 'user_id', 'id', name=op.f('pk_users_roles'))
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('users_roles')
    op.drop_table('roles_permissions')
    op.drop_table('auth_codes')
    op.drop_table('users')
    op.drop_table('roles')
    op.drop_table('permissions')
    # ### end Alembic commands ###



import click
from click import Group
from litestar.plugins import CLIPlugin

from core.cli import coro
from core.database import session_maker
from core.mail import MailClient
from core.settings import settings
from core.storage import provide_storage

from .loaders import AuthLoader
from .services import (
    provide_auth_service,
    provide_permission_service,
    provide_role_service,
    provide_user_service,
)


class AuthPlugin(CLIPlugin):
    def on_cli_init(self, cli: Group) -> None:
        @cli.group(help="Manage auth, load data with ``load`` command")
        @click.version_option(prog_name="auth")
        def auth(): ...

        @auth.command(help="load auth data")
        @coro
        async def load():
            async with session_maker() as session:
                click.echo("Loading auth data... ")
                loader = AuthLoader(
                    user_service=await provide_user_service(session),
                    role_service=await provide_role_service(session),
                    permission_service=await provide_permission_service(session),
                )

                await loader.clear()
                await loader.load()

        @auth.command(help="load auth data")
        @coro
        async def clear():
            async with session_maker() as session:
                user_service = await provide_user_service(session)
                await user_service.delete_where()
                click.echo("Clear auth data")

        @auth.command(help="Send test mail")
        @click.argument("recipient")
        def send_mail(recipient):
            mail_controller = MailClient(settings.mail_config)
            mail_controller.send([recipient], "test", "test")

        @auth.command(help="Set user password")
        @coro
        @click.argument("email")
        @click.argument("password")
        async def set_user_password(email, password):
            async with session_maker() as session:
                mail_controller = MailClient(settings.mail_config)
                async for auth_service in provide_auth_service(
                    db_session=session,
                    mail_client=mail_controller,
                    storage=await provide_storage(),
                    user_service=await provide_user_service(session),
                ):
                    await auth_service.set_user_password(email, password)



from advanced_alchemy.base import UUIDAuditBase
from sqlalchemy import ForeignKey, UniqueConstraint
from sqlalchemy.orm import Mapped, mapped_column


class AuthCodeModel(UUIDAuditBase):
    __tablename__ = "auth_codes"

    code: Mapped[str]
    action: Mapped[str]
    user_id = mapped_column(ForeignKey("users.id", ondelete="CASCADE"))

    __table_args__ = (UniqueConstraint("user_id", "action", name="unique_user_action"),)



from litestar import Controller, get, post
from litestar.di import Provide
from litestar.exceptions import HTTPException
import msgspec

from auth.guards import is_debug_guard
from auth.loaders import AuthLoader
from auth.services import (
    AuthCodeAction,
    AuthService,
    PermissionService,
    RoleService,
    UserService,
    provide_auth_service,
    provide_permission_service,
    provide_role_service,
    provide_user_service,
)


class E2EAuthCodeSchemeResponse(msgspec.Struct):
    code: str


class E2EResetResponse(msgspec.Struct):
    message: str


class E2EController(Controller):
    guards = [is_debug_guard]
    path = "e2e"
    dependencies = {
        "user_service": Provide(provide_user_service),
        "role_service": Provide(provide_role_service),
        "permission_service": Provide(provide_permission_service),
        "auth_service": Provide(provide_auth_service),
    }

    @get(path="/auth-code/verify-email/{email:str}", exclude_from_auth=True)
    async def get_activate_code_for_verify_email(
        self,
        email: str,
        auth_service: AuthService,
    ) -> E2EAuthCodeSchemeResponse:
        auth_code = await auth_service.get_user_auth_code(email, AuthCodeAction.VERIFY_EMAIL)
        if auth_code is None:
            raise HTTPException(detail="User not found", status_code=404)
        return E2EAuthCodeSchemeResponse(code=auth_code.code)

    @get(path="/auth-code/reset-password/{email:str}", exclude_from_auth=True)
    async def get_activate_code_for_reset_password(
        self,
        email: str,
        auth_service: AuthService,
    ) -> E2EAuthCodeSchemeResponse:
        auth_code = await auth_service.get_user_auth_code(email, AuthCodeAction.RESET_PASSWORD)
        if auth_code is None:
            raise HTTPException(detail="User not found", status_code=404)
        return E2EAuthCodeSchemeResponse(code=auth_code.code)

    @post("/reset", exclude_from_auth=True)
    async def reset(
        self,
        user_service: UserService,
        role_service: RoleService,
        permission_service: PermissionService,
    ) -> E2EResetResponse:
        loader = AuthLoader(
            user_service=user_service,
            role_service=role_service,
            permission_service=permission_service,
        )

        await loader.clear()
        await loader.load()
        return E2EResetResponse(message="ok")



from dataclasses import dataclass
from enum import StrEnum

__all__ = ["AuthData", "PermissionMap", "RoleData", "UserData", "roles_map"]


class PermissionMap(StrEnum):
    CREATE_USERS = "create users"
    CREATE_ROLES = "create roles"
    CREATE_PERMISSIONS = "create permissions"
    DELETE_USERS = "delete users"
    DELETE_ROLES = "delete roles"
    DELETE_PERMISSIONS = "delete permissions"
    EDIT_USERS = "edit users"
    EDIT_ROLES = "edit roles"
    EDIT_PERMISSIONS = "edit permissions"
    GET_USERS = "get users"
    GET_ROLES = "get roles"
    GET_PERMISSIONS = "get permissions"


@dataclass
class RoleData:
    name: str
    permissions: list[PermissionMap]


@dataclass
class UserData:
    email: str
    password: str
    roles: list[RoleData]
    is_email_verified: bool
    is_enabled: bool


roles_map = {
    "admin": RoleData(
        name="admin",
        permissions=[
            PermissionMap.CREATE_USERS,
            PermissionMap.CREATE_ROLES,
            PermissionMap.CREATE_PERMISSIONS,
            PermissionMap.EDIT_USERS,
            PermissionMap.EDIT_ROLES,
            PermissionMap.EDIT_PERMISSIONS,
            PermissionMap.GET_USERS,
            PermissionMap.GET_ROLES,
            PermissionMap.GET_PERMISSIONS,
            PermissionMap.DELETE_USERS,
            PermissionMap.DELETE_ROLES,
            PermissionMap.DELETE_PERMISSIONS,
        ],
    ),
    "moderator": RoleData(
        name="moderator",
        permissions=[
            PermissionMap.DELETE_USERS,
            PermissionMap.DELETE_ROLES,
            PermissionMap.DELETE_PERMISSIONS,
        ],
    ),
    "user": RoleData(
        name="user",
        permissions=[PermissionMap.GET_ROLES, PermissionMap.GET_PERMISSIONS],
    ),
    "manager": RoleData(
        name="manager",
        permissions=[
            PermissionMap.CREATE_USERS,
            PermissionMap.CREATE_ROLES,
            PermissionMap.CREATE_PERMISSIONS,
            PermissionMap.EDIT_USERS,
            PermissionMap.EDIT_ROLES,
            PermissionMap.EDIT_PERMISSIONS,
            PermissionMap.GET_USERS,
            PermissionMap.GET_ROLES,
            PermissionMap.GET_PERMISSIONS,
        ],
    ),
}


class AuthData:
    def __init__(self):
        self.permissions = [
            PermissionMap.CREATE_USERS,
            PermissionMap.CREATE_ROLES,
            PermissionMap.CREATE_PERMISSIONS,
            PermissionMap.EDIT_USERS,
            PermissionMap.EDIT_ROLES,
            PermissionMap.EDIT_PERMISSIONS,
            PermissionMap.GET_USERS,
            PermissionMap.GET_ROLES,
            PermissionMap.GET_PERMISSIONS,
            PermissionMap.DELETE_USERS,
            PermissionMap.DELETE_ROLES,
            PermissionMap.DELETE_PERMISSIONS,
        ]
        self.roles = roles_map.values()
        self.users = [
            UserData(
                email="admin@mail.com",
                password="password",
                is_enabled=True,
                is_email_verified=True,
                roles=[roles_map["admin"]],
            ),
            UserData(
                email="supervisor@mail.com",
                password="password",
                is_enabled=True,
                is_email_verified=True,
                roles=[roles_map["moderator"], roles_map["manager"]],
            ),
            UserData(
                email="manager@mail.com",
                password="password",
                is_enabled=True,
                is_email_verified=True,
                roles=[roles_map["manager"]],
            ),
            UserData(
                email="user@mail.com",
                password="password",
                is_enabled=True,
                is_email_verified=True,
                roles=[roles_map["user"]],
            ),
            UserData(
                email="notenabled@mail.com",
                password="password",
                is_enabled=False,
                is_email_verified=True,
                roles=[roles_map["user"]],
            ),
            UserData(
                email="notverified@mail.com",
                password="password",
                is_enabled=True,
                is_email_verified=False,
                roles=[roles_map["user"], roles_map["manager"]],
            ),
            UserData(
                email="reset-password@mail.com",
                password="password",
                is_enabled=True,
                is_email_verified=True,
                roles=[roles_map["user"], roles_map["manager"]],
            ),
        ]


auth_data = AuthData()



from litestar.connection import ASGIConnection
from litestar.exceptions import HTTPException
from litestar.handlers.base import BaseRouteHandler
from litestar.status_codes import HTTP_403_FORBIDDEN


def is_debug_guard(connection: ASGIConnection, _: BaseRouteHandler) -> None:
    if not connection.app.debug:
        raise HTTPException(detail="only in debug mode", status_code=HTTP_403_FORBIDDEN)



"""update auth code model, add action

Revision ID: fafdb22e5a16
Revises: 491aaa1a58f6
Create Date: 2026-01-23 11:57:52.119578

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
import advanced_alchemy


# revision identifiers, used by Alembic.
revision: str = 'fafdb22e5a16'
down_revision: Union[str, Sequence[str], None] = '491aaa1a58f6'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('auth_codes', sa.Column('action', sa.String(), nullable=False))
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('auth_codes', 'action')
    # ### end Alembic commands ###



from typing import Annotated

from advanced_alchemy.extensions.litestar.dto import SQLAlchemyDTO, SQLAlchemyDTOConfig
from advanced_alchemy.extensions.litestar.providers import create_service_dependencies
from advanced_alchemy.filters import FilterTypes
from advanced_alchemy.service import OffsetPagination
from litestar import delete, get, post
from litestar.controller import Controller
from litestar.params import Dependency
import msgspec

from ..models import RoleModel
from ..services import RoleService


class RoleCreateRequest(msgspec.Struct):
    name: str


class RoleDTO(SQLAlchemyDTO[RoleModel]):
    config = SQLAlchemyDTOConfig()


class RoleController(Controller):
    dependencies = create_service_dependencies(
        RoleService,
        key="service",
        filters={
            "created_at": True,
            "updated_at": True,
        },
    )
    return_dto = RoleDTO

    @get(operation_id="ListRoles", path="/roles")
    async def list_roles(
        self,
        service: RoleService,
        filters: Annotated[list[FilterTypes], Dependency(skip_validation=True)],
    ) -> OffsetPagination[RoleModel]:
        results, total = await service.list_and_count(*filters)
        return service.to_schema(data=results, total=total, filters=filters)

    @post(operation_id="CreateRole", path="/roles")
    async def create_role(self, service: RoleService, data: RoleCreateRequest) -> RoleModel:
        return await service.create(RoleModel(**msgspec.to_builtins(data)), auto_commit=True)

    @delete(operation_id="DeleteRole", path="/roles/{item_id:str}")
    async def delete_role(self, service: RoleService, item_id: str) -> None:
        await service.delete(item_id, auto_commit=True)



from core.mail import MailClient, MailSettings


class TestMailClient(MailClient):
    def __init__(self, settings: MailSettings):
        super().__init__(settings)
        self.messages = []

    def send(self, recipients: list[str], subject: str, body: str):
        self.messages.append((recipients, subject, body))

    def reset(self):
        self.messages = []



from advanced_alchemy.repository import SQLAlchemyAsyncRepository
from advanced_alchemy.service import SQLAlchemyAsyncRepositoryService
from sqlalchemy.ext.asyncio import AsyncSession

from ..models import RoleModel

__all__ = [
    "RoleService",
    "provide_role_service",
]


class RoleService(SQLAlchemyAsyncRepositoryService):
    class Repository(SQLAlchemyAsyncRepository[RoleModel]):
        model_type = RoleModel

    repository_type = Repository


async def provide_role_service(db_session: AsyncSession) -> RoleService:
    return RoleService(session=db_session)



from litestar import Router

from .account_controller import AccountController
from .e2e_controller import E2EController
from .permission_controller import PermissionController
from .role_controller import RoleController
from .user_controller import UserController

auth_router = Router(
    path="/api/auth",
    route_handlers=[UserController, RoleController, PermissionController, AccountController, E2EController],
)

__all__ = ["auth_router"]



from litestar.status_codes import HTTP_200_OK, HTTP_201_CREATED, HTTP_204_NO_CONTENT
from litestar.testing import AsyncTestClient

from tests.factories import PermissionFactory


async def test_get_permissions(admin_api_client: AsyncTestClient):
    response = await admin_api_client.get("/api/auth/permissions")
    assert response.status_code == HTTP_200_OK


async def test_create_permission(admin_api_client: AsyncTestClient, permissions_fixture, permission_service):
    prev_items_count = await permission_service.count()

    response = await admin_api_client.post("/api/auth/permissions", json={"name": "allow-edit"})
    assert response.status_code == HTTP_201_CREATED

    items_count = await permission_service.count()
    assert items_count == prev_items_count + 1

    # teardown
    await permission_service.delete(response.json()["id"])


async def test_remove_permission(
    admin_api_client: AsyncTestClient,
    permissions_fixture,
    permission_service,
):
    permissions_count = await permission_service.count()

    permission_to_remove = await permission_service.create(PermissionFactory.build(), auto_commit=True)
    response = await admin_api_client.delete(f"/api/auth/permissions/{permission_to_remove.id}")
    assert response.status_code == HTTP_204_NO_CONTENT
    assert permissions_count == await permission_service.count()



from faker import Faker
from polyfactory.factories.sqlalchemy_factory import SQLAlchemyFactory
from polyfactory.fields import Use

from auth.models import PermissionModel, RoleModel, UserModel

faker = Faker()


class UserFactory(SQLAlchemyFactory[UserModel]):
    email = Use(faker.unique.email)


class RoleFactory(SQLAlchemyFactory[RoleModel]):
    name = Use(faker.unique.word)


class PermissionFactory(SQLAlchemyFactory[PermissionModel]):
    name = Use(faker.unique.word)



import asyncio
from collections.abc import AsyncGenerator
from typing import Any, AsyncGenerator

from advanced_alchemy.base import UUIDAuditBase
from litestar import Litestar
from litestar.di import Provide
import pytest
import sqlalchemy
from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker, create_async_engine

from auth import auth_router
from auth.middleware import JWTAuthMiddleware
from core.database import sqlalchemy_plugin
from core.settings import settings
from core.storage import Storage, close_storage, provide_storage
from tests.integrations import TestMailClient


@pytest.fixture(scope="session")
def mail_client():
    return TestMailClient(settings.mail_config)


@pytest.fixture(scope="session")
def application(db_session, mail_client):
    def provide_mail_client():
        return mail_client

    application = Litestar(
        route_handlers=[auth_router],
        on_shutdown=[close_storage],
        plugins=[sqlalchemy_plugin],
        dependencies={
            "mail_client": Provide(provide_mail_client),
            "storage": Provide(provide_storage),
        },
        debug=settings.debug,
        middleware=[JWTAuthMiddleware],
    )
    yield application


@pytest.fixture(scope="session")
def event_loop():
    return asyncio.get_event_loop()


@pytest.fixture(scope="session")
async def db_session() -> AsyncGenerator[AsyncSession, Any]:
    engine = create_async_engine(url=settings.db_url)

    metadata = UUIDAuditBase.registry.metadata
    async with engine.begin() as conn:
        await conn.run_sync(metadata.drop_all)
        await conn.run_sync(metadata.create_all)

    # create async engine
    async with async_sessionmaker(engine, expire_on_commit=False)() as session:
        try:
            yield session
        except Exception as e:
            await session.rollback()
            raise e
        finally:
            await session.close()

    # clear database
    async with engine.begin() as conn:
        meta = sqlalchemy.MetaData()
        await conn.run_sync(meta.reflect)
        await conn.run_sync(meta.drop_all)


@pytest.fixture(scope="function")
async def storage() -> AsyncGenerator[Storage, Any]:
    storage = await provide_storage()
    await storage.connect()
    yield storage
    await storage.disconnect()



from litestar import Litestar
from litestar.di import Provide

from auth.api import auth_router
from auth.middleware import JWTAuthMiddleware
from auth.plugin import AuthPlugin
from core.database import sqlalchemy_plugin
from core.mail import provide_mail_client
from core.openapi import openapi_config
from core.settings import settings
from core.storage import close_storage, provide_storage


def create_app():
    app = Litestar(
        route_handlers=[auth_router],
        middleware=[JWTAuthMiddleware],
        plugins=[sqlalchemy_plugin, AuthPlugin()],
        openapi_config=openapi_config,
        debug=settings.debug,
        on_shutdown=[close_storage],
        dependencies={
            "storage": Provide(provide_storage),
            "mail_client": Provide(provide_mail_client),
        },
    )
    return app



from enum import StrEnum
import random
from typing import Any, AsyncGenerator
from uuid import UUID, uuid4

import advanced_alchemy
from advanced_alchemy.repository import SQLAlchemyAsyncRepository
import bcrypt
import jwt
from litestar.exceptions import NotAuthorizedException
import msgspec
from nats.js.errors import KeyNotFoundError
from sqlalchemy.ext.asyncio import AsyncSession

from core.mail import MailClient
from core.settings import settings
from core.storage import Storage

from ..models import AuthCodeModel, UserModel
from .user_service import UserService

__all__ = [
    # services
    "UserService",
    "AuthService",
    "provide_auth_service",
    # dto
    "LoginRequestDTO",
    "LogoutRequestDTO",
    "SignUpRequestDTO",
    "AccountCacheDTO",
    "AccountCacheSessionDTO",
    "AccountDTO",
    "AccountUserDTO",
    "ConfirmResetPasswordRequestDTO",
    # constants
    "API_KEY_HEADER",
    "TOKEN_PREFIX",
    # errors
    "UserEmailNotVerifiedError",
    "WrongAuthCodeError",
    "EmailIsAlreadyUsedError",
    "InvalidPasswordError",
    "InvalidEmailError",
    "DecodeTokenError",
    "UserNotEnabledError",
    "AuthCodeAction",
    "StartResetPasswordEmailNotFoundError",
    "ConfirmResetPasswordWrongCodeError",
]


class AuthCodeAction(StrEnum):
    VERIFY_EMAIL = "verify_email"
    RESET_PASSWORD = "reset_password"


API_KEY_HEADER = "Authorization"
TOKEN_PREFIX = "Bearer"


class EmailIsAlreadyUsedError(Exception): ...


class StartResetPasswordEmailNotFoundError(Exception): ...


class ConfirmResetPasswordWrongCodeError(Exception): ...


class InvalidPasswordError(Exception): ...


class InvalidEmailError(Exception): ...


class UserNotEnabledError(Exception): ...


class UserEmailNotVerifiedError(Exception): ...


class DecodeTokenError(Exception): ...


class WrongAuthCodeError(Exception): ...


class AccountRoleDTO(msgspec.Struct):
    id: UUID
    name: str


class AccountUserDTO(msgspec.Struct):
    id: UUID
    email: str
    is_email_verified: bool
    is_enabled: bool
    roles: list[AccountRoleDTO] = msgspec.field(default_factory=list)


class AccountDTO(msgspec.Struct):
    token: str
    session_id: UUID
    user: AccountUserDTO


class TokenAccountDataDTO(msgspec.Struct):
    session_id: UUID
    user: AccountUserDTO


class ConfirmResetPasswordRequestDTO(msgspec.Struct):
    code: str
    password: str
    device: str


class LoginRequestDTO(msgspec.Struct):
    email: str
    password: str
    device: str


class LogoutRequestDTO(msgspec.Struct):
    user_id: UUID
    session_id: UUID
    device: str


class SignUpRequestDTO(msgspec.Struct):
    email: str
    password: str


class AccountCacheSessionDTO(msgspec.Struct):
    device: str
    session_id: UUID


class AccountCacheDTO(msgspec.Struct):
    sessions: list[AccountCacheSessionDTO]
    user: AccountUserDTO


class LoginResponseDTO(AccountDTO): ...


class ActivateUserResponseDTO(AccountDTO): ...


class AuthCodeRepository(SQLAlchemyAsyncRepository[AuthCodeModel]):
    model_type = AuthCodeModel


class AuthService:
    def __init__(
        self,
        session: AsyncSession,
        mail_client: MailClient,
        storage: Storage,
        user_service: UserService,
    ):
        self.session = session
        self.mail_client = mail_client
        self.storage = storage
        self.user_service = user_service
        self.auth_code_repository = AuthCodeRepository(session=session)

    async def signup(self, user: SignUpRequestDTO) -> AuthCodeModel:
        try:
            signup_user = await self.user_service.create(
                UserModel(
                    email=user.email,
                    password=self.hash_password(user.password),
                    is_email_verified=False,
                    is_enabled=True,
                ),
                auto_commit=True,
            )
        except advanced_alchemy.exceptions.DuplicateKeyError:
            raise EmailIsAlreadyUsedError(f"Email {user.email} is already used.")

        auth_code = await self.auth_code_repository.add(
            AuthCodeModel(
                user_id=signup_user.id,
                code=self.generate_activate_code(),
                action=AuthCodeAction.VERIFY_EMAIL,
            ),
            auto_commit=True,
        )

        self.mail_client.send([user.email], "Sign up", auth_code.code)
        return auth_code

    async def enable_user(self, user_id: UUID) -> UserModel:
        return await self.user_service.update(UserModel(id=user_id, is_enabled=True), auto_commit=True)

    async def disable_user(self, user_id: UUID) -> UserModel:
        return await self.user_service.update(UserModel(id=user_id, is_enabled=False), auto_commit=True)

    async def start_reset_password(self, email: str) -> AuthCodeModel:
        try:
            user = await self.user_service.get_one(UserModel.email == email)
        except advanced_alchemy.exceptions.NotFoundError:
            raise StartResetPasswordEmailNotFoundError(f"User with email {email} not found.")

        auth_code, _ = await self.auth_code_repository.get_or_upsert(
            match_fields={
                "user_id": user.id,
                "action": AuthCodeAction.RESET_PASSWORD,
            },
            user_id=user.id,
            action=AuthCodeAction.RESET_PASSWORD,
            code=self.generate_activate_code(),
            auto_commit=True,
        )

        self.mail_client.send([email], "Reset password", auth_code.code)
        return auth_code

    async def confirm_rest_password(self, data: ConfirmResetPasswordRequestDTO) -> LoginResponseDTO:
        auth_code = await self.auth_code_repository.get_one_or_none(
            AuthCodeModel.code == data.code,
            AuthCodeModel.action == AuthCodeAction.RESET_PASSWORD,
        )
        if not auth_code:
            raise ConfirmResetPasswordWrongCodeError(f"Reset password with code {data.code} not found.")

        # if auth code is success, update password and verify user email
        user = await self.user_service.update(
            UserModel(
                id=auth_code.user_id,
                password=self.hash_password(data.password),
                is_email_verified=True,
            ),
            auto_commit=True,
        )
        return await self._add_account_session(user, data.device)

    async def set_user_password(self, email: str, new_password: str) -> UserModel:
        user = await self.user_service.get_one(UserModel.email == email)

        return await self.user_service.update(
            UserModel(id=user.id, password=self.hash_password(new_password)), auto_commit=True
        )

    async def verify_user_email(self, code: str, device: str | None) -> LoginResponseDTO:
        auth_code = await self.auth_code_repository.get_one_or_none(
            AuthCodeModel.code == code,
            AuthCodeModel.action == AuthCodeAction.VERIFY_EMAIL,
        )
        if not auth_code:
            raise WrongAuthCodeError

        user = await self.user_service.update(
            UserModel(id=auth_code.user_id, is_email_verified=True),
            auto_commit=True,
        )
        return await self._add_account_session(user, device)

    async def login(self, login_data: LoginRequestDTO) -> LoginResponseDTO:
        try:
            login_user = await self.user_service.get_one(UserModel.email == login_data.email)

        except advanced_alchemy.exceptions.NotFoundError:
            raise InvalidEmailError(f"Email {login_data.email} not found")

        if not login_user.is_enabled:
            raise UserNotEnabledError(f"User {login_data.email} not enabled")

        if not login_user.is_email_verified:
            raise UserEmailNotVerifiedError(f"User {login_data.email} not verified")

        if not self._check_password(login_data.password, login_user.password):
            raise InvalidPasswordError

        return await self._add_account_session(login_user, login_data.device)

    async def _add_account_session(self, user: UserModel, device: str) -> LoginResponseDTO:
        login_session = AccountCacheSessionDTO(
            session_id=uuid4(),
            device=device,
        )

        account_user = AccountUserDTO(
            id=user.id,
            email=user.email,
            is_email_verified=user.is_email_verified,
            is_enabled=user.is_enabled,
            roles=[AccountRoleDTO(id=role.id, name=role.name) for role in user.roles],
        )

        try:
            cached_account = await self.storage.get("sessions", str(user.id), model_type=AccountCacheDTO)
            cached_account.sessions.append(login_session)
        except KeyNotFoundError:
            cached_account = AccountCacheDTO(user=account_user, sessions=[login_session])

        jwt_token = self.encode_token(
            TokenAccountDataDTO(
                session_id=login_session.session_id,
                user=account_user,
            )
        )
        await self.storage.save("sessions", str(cached_account.user.id), cached_account)

        return LoginResponseDTO(
            token=jwt_token,
            session_id=login_session.session_id,
            user=account_user,
        )

    async def logout(self, logout_request: LogoutRequestDTO) -> None:
        cached_user = await self.storage.get("sessions", str(logout_request.user_id), model_type=AccountCacheDTO)
        cached_user.sessions = [
            session
            for session in cached_user.sessions
            if not (session.session_id == logout_request.session_id and session.device == logout_request.device)
        ]
        await self.storage.save("sessions", str(logout_request.user_id), cached_user)

    async def check_token(self, token: str, user_agent: str) -> AccountDTO:
        try:
            account = await self.get_account(token)
        except DecodeTokenError:
            raise NotAuthorizedException("Decode token fail")

        await self.check_session(account, user_agent)
        return account

    async def check_session(self, account: AccountDTO, device: str):
        try:
            cached_account = await self.storage.get("sessions", str(account.user.id), model_type=AccountCacheDTO)
            for session in cached_account.sessions:
                if session.session_id == account.session_id:
                    if session.device != device:
                        raise NotAuthorizedException("Wrong device")
                    return
            raise NotAuthorizedException("Session not found")
        except KeyNotFoundError:
            raise NotAuthorizedException()

    async def get_user_auth_code(self, email: str, action: str) -> AuthCodeModel | None:
        user = await self.user_service.get_one_or_none(UserModel.email == email)
        return await self.auth_code_repository.get_one_or_none(
            AuthCodeModel.user_id == user.id, AuthCodeModel.action == action
        )

    @staticmethod
    def _check_password(password: str, hashed_password: str) -> bool:
        return bcrypt.checkpw(password.encode("utf-8"), hashed_password.encode("utf-8"))

    @staticmethod
    def hash_password(password: str) -> str:
        return bcrypt.hashpw(password.encode("utf8"), bcrypt.gensalt()).decode("utf8")

    async def get_account(self, token: str) -> AccountDTO:
        try:
            account_data = self.decode_token(token)
            token_account_data = msgspec.convert(account_data, type=TokenAccountDataDTO)
            return AccountDTO(
                token=token,
                session_id=token_account_data.session_id,
                user=token_account_data.user,
            )
        except Exception:
            raise DecodeTokenError

    @staticmethod
    def encode_token(token_account_data: TokenAccountDataDTO) -> str:
        return jwt.encode(msgspec.to_builtins(token_account_data), settings.jwt_secret, algorithm="HS256")

    @staticmethod
    def decode_token(token: str) -> dict[str, Any]:
        try:
            return jwt.decode(token, settings.jwt_secret, algorithms=["HS256"])
        except jwt.exceptions.InvalidSignatureError:
            raise DecodeTokenError

    @staticmethod
    def generate_activate_code() -> str:
        return "".join(str(random.randint(0, 9)) for _ in range(8))


async def provide_auth_service(
    db_session: AsyncSession,
    mail_client: MailClient,
    storage: Storage,
    user_service: UserService,
) -> AsyncGenerator[AuthService, None]:
    yield AuthService(
        session=db_session,
        mail_client=mail_client,
        storage=storage,
        user_service=user_service,
    )



from auth.data import auth_data
from auth.models import PermissionModel, RoleModel, UserModel
from auth.services import AuthService, PermissionService, RoleService, UserService


class AuthLoader:
    def __init__(
        self,
        user_service: UserService,
        role_service: RoleService,
        permission_service: PermissionService,
    ):
        self.user_service = user_service
        self.role_service = role_service
        self.permission_service = permission_service
        self.cache = {"permissions": {}, "roles": {}}
        self.auth_data = auth_data

    async def load(self):
        await self.load_permissions()
        await self.load_roles()
        await self.load_users()

    async def load_permissions(self):
        permissions = []
        for permission_name in self.auth_data.permissions:
            permission = PermissionModel(name=permission_name.value)
            self.cache["permissions"][permission_name] = permission
            permissions.append(permission)

        await self.permission_service.create_many(
            permissions,
            auto_commit=True,
        )

    async def load_roles(self):
        roles = []
        for role_data in self.auth_data.roles:
            role = RoleModel(
                name=role_data.name,
                permissions=[
                    self.cache["permissions"][permission_name.value] for permission_name in role_data.permissions
                ],
            )
            roles.append(role)
            self.cache["roles"][role_data.name] = role
        await self.role_service.create_many(roles, auto_commit=True)

    async def load_users(self):
        await self.user_service.create_many(
            [
                UserModel(
                    email=user.email,
                    password=AuthService.hash_password(user.password),
                    is_enabled=user.is_enabled,
                    is_email_verified=user.is_email_verified,
                    roles=[self.cache["roles"][role.name] for role in user.roles],
                )
                for user in self.auth_data.users
            ],
            auto_commit=True,
        )

    async def clear(self):
        await self.permission_service.delete_where(auto_commit=True)
        await self.role_service.delete_where(auto_commit=True)
        await self.user_service.delete_where(auto_commit=True)



from typing import TypeVar

from faststream import FastStream
from faststream.nats import NatsBroker
from litestar import Litestar
import msgspec

from .settings import StorageSettings, settings

T = TypeVar("T")


class Storage:
    def __init__(self, config: StorageSettings):
        self.broker = NatsBroker(servers=config.url)
        self.app = FastStream(self.broker)
        self.settings = config
        self.buckets = {}
        self.is_connected = False

    async def connect(self):
        if not self.is_connected:
            await self.broker.connect()
            for bucket in self.settings.buckets:
                await self.init_bucket(bucket)
            self.is_connected = True

    async def init_bucket(self, name: str):
        self.buckets[name] = await self.broker.key_value(name)

    async def save(self, bucket: str, key: str, data: msgspec.Struct):
        await self.buckets[bucket].put(key, msgspec.json.encode(data))

    async def get(self, bucket: str, key: str, model_type: T) -> T:
        data = await self.buckets[bucket].get(key)
        return msgspec.json.decode(data.value, type=model_type)

    async def delete(self, bucket: str, key: str):
        await self.buckets[bucket].delete(key)

    async def disconnect(self):
        await self.broker.stop()
        self.is_connected = False


async def provide_storage() -> Storage:
    storage = Storage(settings.storage_config)
    await storage.connect()
    return storage


async def close_storage(app: Litestar) -> None:
    storage: Storage = await app.dependencies.get("storage")()
    await storage.disconnect()



from advanced_alchemy.base import UUIDAuditBase
from sqlalchemy.orm import Mapped, mapped_column, relationship


class UserModel(UUIDAuditBase):
    __tablename__ = "users"

    email: Mapped[str] = mapped_column(unique=True)
    password: Mapped[str]
    is_email_verified: Mapped[bool]
    is_enabled: Mapped[bool]

    roles = relationship(
        "RoleModel",
        secondary="users_roles",
        lazy="selectin",
    )



from uuid import UUID

from litestar.status_codes import HTTP_200_OK, HTTP_201_CREATED, HTTP_400_BAD_REQUEST, HTTP_401_UNAUTHORIZED
from litestar.testing import AsyncTestClient
import msgspec
import pytest_asyncio

from auth.api.account_controller import StartResetPasswordRequestScheme
from auth.models import AuthCodeModel, UserModel
from auth.services import API_KEY_HEADER, TOKEN_PREFIX, AuthService, LoginRequestDTO, SignUpRequestDTO
from auth.services.auth_service import AuthCodeAction
from tests.factories import UserFactory


class ActivatedDataUserFixture(msgspec.Struct):
    id: UUID
    email: str
    password: str


class ActivatedDataFixture(msgspec.Struct):
    user: ActivatedDataUserFixture
    auth_code: AuthCodeModel


@pytest_asyncio.fixture
async def activated_data(auth_service: AuthService) -> ActivatedDataFixture:
    user_factory_data = UserFactory.build()

    # signup user
    auth_code = await auth_service.signup(
        SignUpRequestDTO(email=user_factory_data.email, password=user_factory_data.password)
    )

    # verify user email
    await auth_service.verify_user_email(auth_code.code, "testclient")

    return ActivatedDataFixture(
        user=ActivatedDataUserFixture(
            id=auth_code.user_id,
            email=user_factory_data.email,
            password=user_factory_data.password,
        ),
        auth_code=auth_code,
    )


async def test_account_signup(http_api_client):
    response = await http_api_client.post(
        "/api/auth/account/signup",
        json={
            "email": "newuser@mail.com",
            "password": "newuserpassword",
        },
    )
    assert response.status_code == HTTP_201_CREATED


async def test_email_is_already_used(http_api_client, activated_data: ActivatedDataFixture):
    response = await http_api_client.post(
        "/api/auth/account/signup",
        json={
            "email": activated_data.user.email,
            "password": "newuserpassword",
        },
    )
    assert response.status_code == HTTP_400_BAD_REQUEST


async def test_account_login(activated_data, http_api_client: AsyncTestClient):
    login_response = await http_api_client.post(
        "/api/auth/account/login",
        json={"email": activated_data.user.email, "password": activated_data.user.password},
    )
    assert login_response.status_code == HTTP_201_CREATED
    login_response_json = login_response.json()
    assert login_response_json["user"]["email"] == activated_data.user.email
    assert login_response_json["user"]["id"] == str(activated_data.auth_code.user_id)


async def test_account_login_wrong_email(activated_data, http_api_client: AsyncTestClient):
    wrong_email = "wrong@email.com"

    login_response = await http_api_client.post(
        "/api/auth/account/login",
        json={"email": wrong_email, "password": activated_data.user.password},
    )
    assert login_response.status_code == HTTP_400_BAD_REQUEST
    assert login_response.json()["detail"] == "Email not found"


async def test_account_login_wrong_password(activated_data, http_api_client: AsyncTestClient):
    login_response = await http_api_client.post(
        "/api/auth/account/login",
        json={"email": activated_data.user.email, "password": "wrong_password"},
    )
    assert login_response.status_code == HTTP_400_BAD_REQUEST
    assert login_response.json()["detail"] == "Invalid password"


async def test_account_email_not_verified(http_api_client: AsyncTestClient, auth_service: AuthService):
    user = UserFactory.build()
    await auth_service.signup(SignUpRequestDTO(email=user.email, password=user.password))

    login_response = await http_api_client.post(
        "/api/auth/account/login",
        json={"email": user.email, "password": user.password},
    )
    assert login_response.status_code == HTTP_400_BAD_REQUEST
    assert login_response.json()["detail"] == "Email not verified"


async def test_account_is_not_enabled(
    http_api_client: AsyncTestClient, auth_service: AuthService, activated_data: ActivatedDataFixture
):
    await auth_service.disable_user(activated_data.auth_code.user_id)

    login_response = await http_api_client.post(
        "/api/auth/account/login",
        json={"email": activated_data.user.email, "password": activated_data.user.password},
    )
    assert login_response.status_code == HTTP_400_BAD_REQUEST
    assert login_response.json()["detail"] == "User is not enabled"


async def test_account_me(
    http_api_client: AsyncTestClient, auth_service: AuthService, activated_data: ActivatedDataFixture
):
    login_data = await auth_service.login(
        LoginRequestDTO(
            email=activated_data.user.email,
            password=activated_data.user.password,
            device="testclient",
        ),
    )

    account_response = await http_api_client.get(
        "/api/auth/account/me",
        headers={API_KEY_HEADER: f"{TOKEN_PREFIX} {login_data.token}"},
    )
    assert account_response.status_code == HTTP_200_OK
    assert activated_data.user.email == account_response.json()["user"]["email"]


async def test_account_logout(http_api_client: AsyncTestClient, auth_service: AuthService, activated_data):
    login_data = await auth_service.login(
        LoginRequestDTO(email=activated_data.user.email, password=activated_data.user.password, device="testclient")
    )
    await http_api_client.post(
        "/api/auth/account/logout",
        headers={API_KEY_HEADER: f"{TOKEN_PREFIX} {login_data.token}"},
    )
    account_response = await http_api_client.get(
        "/api/auth/account/me",
        headers={API_KEY_HEADER: f"{TOKEN_PREFIX} {login_data.token}"},
    )
    assert account_response.status_code == HTTP_401_UNAUTHORIZED


async def test_verify_account_email(http_api_client: AsyncTestClient, auth_service: AuthService):
    user = UserFactory.build()
    auth_code = await auth_service.signup(SignUpRequestDTO(email=user.email, password=user.password))
    activate_response = await http_api_client.post("/api/auth/account/verify-email", json={"code": auth_code.code})
    assert activate_response.status_code == HTTP_201_CREATED

    login_data = await auth_service.login(
        LoginRequestDTO(
            email=user.email,
            password=user.password,
            device="testclient",
        )
    )

    account_data = await auth_service.get_account(login_data.token)
    assert account_data.user.email == user.email


async def test_not_authorized_with_wrong_tokenkey(
    http_api_client: AsyncTestClient, auth_service: AuthService, activated_data: ActivatedDataFixture
):
    login_data = await auth_service.login(
        LoginRequestDTO(
            email=activated_data.user.email,
            password=activated_data.user.password,
            device="testclient",
        )
    )

    account_response = await http_api_client.get(
        "/api/auth/account/me",
        headers={API_KEY_HEADER: f"{TOKEN_PREFIX} {login_data.token}_invalid"},
    )
    assert account_response.status_code == HTTP_401_UNAUTHORIZED


async def test_rest_password_success(
    http_api_client: AsyncTestClient, auth_service: AuthService, activated_data: ActivatedDataFixture
):
    new_password = "new_password"

    response = await http_api_client.post(
        "/api/auth/account/start-reset-password",
        json={
            "email": activated_data.user.email,
        },
    )
    assert response.status_code == HTTP_201_CREATED

    auth_code = await auth_service.auth_code_repository.get_one_or_none(
        AuthCodeModel.user_id == activated_data.user.id,
        AuthCodeModel.action == AuthCodeAction.RESET_PASSWORD,
    )

    response = await http_api_client.post(
        "/api/auth/account/confirm-reset-password",
        json={
            "password": new_password,
            "code": auth_code.code,
        },
    )

    assert response.status_code == HTTP_201_CREATED


async def test_start_reset_password_wrong_email(
    http_api_client: AsyncTestClient, auth_service: AuthService, activated_data: ActivatedDataFixture
):
    response = await http_api_client.post(
        "/api/auth/account/start-reset-password",
        json={
            "email": "wrong@mail.com",
        },
    )
    assert response.status_code == HTTP_400_BAD_REQUEST
    assert response.json() == {
        "detail": "Start reset password failed, email not found",
        "status_code": HTTP_400_BAD_REQUEST,
    }


async def test_confirm_reset_password_wrong_code(
    http_api_client: AsyncTestClient,
    auth_service: AuthService,
    activated_data: ActivatedDataFixture,
):
    await auth_service.start_reset_password(activated_data.user.email)
    response = await http_api_client.post(
        "/api/auth/account/confirm-reset-password", json={"password": "new_password", "code": "00000000"}
    )

    assert response.status_code == HTTP_400_BAD_REQUEST
    assert response.json() == {
        "detail": "Confirm reset password failed, wrong code",
        "status_code": HTTP_400_BAD_REQUEST,
    }



from litestar.connection import ASGIConnection
from litestar.exceptions import NotAuthorizedException
from litestar.middleware import (
    AbstractAuthenticationMiddleware,
    AuthenticationResult,
)

from core.storage import Storage

from .services import provide_auth_service, provide_user_service
from .services.auth_service import TOKEN_PREFIX, AuthService


class JWTAuthMiddleware(AbstractAuthenticationMiddleware):
    async def authenticate_request(self, connection: ASGIConnection) -> AuthenticationResult:
        auth_header = connection.headers.get("Authorization")
        if not auth_header:
            raise NotAuthorizedException()

        token = auth_header.replace(f"{TOKEN_PREFIX} ", "")
        auth_service: AuthService = await self._get_auth_service(connection)

        return AuthenticationResult(
            user=await auth_service.check_token(token, connection.headers.get("User-Agent")),
            auth=token,
        )

    @staticmethod
    async def _get_auth_service(connection: ASGIConnection) -> AuthService:
        storage: Storage = await connection.app.dependencies.get("storage")()
        db_session = await connection.app.dependencies.get("db_session")(
            state=connection.app.state,
            scope=connection.scope,
        )
        mail_client = await connection.app.dependencies.get("mail_client")()
        user_service = await provide_user_service(db_session=db_session)
        async for auth_service in provide_auth_service(
            db_session=db_session,
            mail_client=mail_client,
            storage=storage,
            user_service=user_service,
        ):
            return auth_service



from typing import Annotated

from advanced_alchemy.extensions.litestar.dto import SQLAlchemyDTO, SQLAlchemyDTOConfig
from advanced_alchemy.extensions.litestar.providers import create_service_dependencies
from advanced_alchemy.filters import FilterTypes
from advanced_alchemy.service import OffsetPagination
from litestar import delete, get, post
from litestar.controller import Controller
from litestar.params import Dependency
import msgspec

from ..models import UserModel
from ..services import UserService


class UserCreateRequest(msgspec.Struct):
    email: str
    password: str
    is_email_verified: bool
    is_enabled: bool


class UserDTO(SQLAlchemyDTO[UserModel]):
    config = SQLAlchemyDTOConfig(exclude={"password"})


class UserController(Controller):
    dependencies = create_service_dependencies(
        UserService,
        key="service",
        filters={
            "created_at": True,
            "updated_at": True,
            "sort_field": "email",
            "search": "email",
        },
    )
    return_dto = UserDTO

    @get(operation_id="ListUsers", path="/users")
    async def list_users(
        self,
        service: UserService,
        filters: Annotated[list[FilterTypes], Dependency(skip_validation=True)],
    ) -> OffsetPagination[UserModel]:
        results, total = await service.list_and_count(*filters)
        return service.to_schema(data=results, total=total, filters=filters)

    @post(operation_id="CreateUser", path="/users")
    async def create_user(self, service: UserService, data: UserCreateRequest) -> UserModel:
        return await service.create(UserModel(**msgspec.to_builtins(data)), auto_commit=True)

    @delete(operation_id="DeleteUser", path="/users/{item_id:str}")
    async def delete_user(self, service: UserService, item_id: str) -> None:
        await service.delete(item_id, auto_commit=True)



from typing import AsyncIterator

from litestar import Litestar
from litestar.testing import AsyncTestClient
import pytest

from tests.factories import UserFactory
from auth.services import AuthService, LoginRequestDTO, SignUpRequestDTO


@pytest.fixture(scope="session")
async def http_api_client(application) -> AsyncIterator[AsyncTestClient[Litestar]]:
    async with AsyncTestClient(app=application) as client:
        yield client


@pytest.fixture(scope="function")
async def admin_api_client(
    http_api_client: AsyncTestClient, auth_service: AuthService
) -> AsyncIterator[AsyncTestClient[Litestar]]:
    user = UserFactory.build()
    auth_code = await auth_service.signup(SignUpRequestDTO(email=user.email, password=user.password))
    await auth_service.verify_user_email(auth_code.code, "testclient")
    login_data = await auth_service.login(
        LoginRequestDTO(email=user.email, password=user.password, device="testclient")
    )
    http_api_client.headers["Authorization"] = f"Bearer {login_data.token}"
    yield http_api_client



from pydantic import Field
from pydantic_settings import BaseSettings


class StorageSettings(BaseSettings):
    url: str = Field(default="nats://localhost:4222")
    buckets: list[str] = Field()

    class Config:
        env_prefix = "STORAGE_"


class MailSettings(BaseSettings):
    host: str = Field(default="smtp.yandex.com")
    port: int = Field(default=587)
    login: str = Field(default="info@service-laboratory.online")
    password: str = Field(default="superPassword")

    class Config:
        env_prefix = "MAIL_"


class OpenApiSettings(BaseSettings):
    title: str = Field(deafult="title")
    version: str = Field(deafult="0.1.2")
    path: str = Field(deafult="/api/docs")

    class Config:
        env_prefix = "OPEN_API_"


class Settings(BaseSettings):
    debug: bool = Field(default=False)
    jwt_secret: str = Field(default="secret_for_jwt")

    # database config
    db_url: str = Field(default="postgresql+asyncpg://user:password@localhost:5432/db")
    db_create_all: bool = Field(default=False)

    # services config
    open_api_config: OpenApiSettings = Field(default_factory=OpenApiSettings)
    mail_config: MailSettings = Field(default_factory=MailSettings)
    storage_config: StorageSettings = Field(default_factory=StorageSettings)


settings = Settings()



from advanced_alchemy.base import UUIDAuditBase
from sqlalchemy.orm import Mapped, mapped_column


class PermissionModel(UUIDAuditBase):
    __tablename__ = "permissions"

    name: Mapped[str] = mapped_column(unique=True)



from advanced_alchemy.repository import SQLAlchemyAsyncRepository
from advanced_alchemy.service import SQLAlchemyAsyncRepositoryService
from sqlalchemy.ext.asyncio import AsyncSession

from ..models import PermissionModel

__all__ = ["PermissionService", "provide_permission_service"]


class PermissionService(SQLAlchemyAsyncRepositoryService):
    class Repository(SQLAlchemyAsyncRepository[PermissionModel]):
        model_type = PermissionModel

    repository_type = Repository


async def provide_permission_service(db_session: AsyncSession) -> PermissionService:
    return PermissionService(session=db_session)



from litestar.status_codes import HTTP_200_OK, HTTP_201_CREATED, HTTP_204_NO_CONTENT

from tests.factories import UserFactory


async def test_get_users(admin_api_client):
    response = await admin_api_client.get("/api/auth/users")
    assert response.status_code == HTTP_200_OK


async def test_create_user(admin_api_client, users_fixture, user_service):
    prev_users_count = await user_service.count()

    user_to_create = {
        "email": "user-to-create@mail.com",
        "password": "superPassword",
        "is_email_verified": False,
        "is_enabled": True,
    }
    response = await admin_api_client.post("/api/auth/users", json=user_to_create)
    assert response.status_code == HTTP_201_CREATED

    users_count = await user_service.count()
    assert users_count == prev_users_count + 1

    # teardown
    await user_service.delete(response.json()["id"])


async def test_remove_user(admin_api_client, users_fixture, user_service):
    users_count = await user_service.count()

    user_to_remove = await user_service.create(UserFactory.build(), auto_commit=True)
    response = await admin_api_client.delete(f"/api/auth/users/{user_to_remove.id}")
    assert response.status_code == HTTP_204_NO_CONTENT
    assert users_count == await user_service.count()



import uuid

from auth.services.auth_service import AccountCacheDTO, AccountCacheSessionDTO, AccountUserDTO
from core.storage import Storage


async def test_storage(storage: Storage) -> None:
    session_value = AccountCacheDTO(
        user=AccountUserDTO(id=uuid.uuid4(), email="user@mail.com", is_enabled=True, is_email_verified=True, roles=[]),
        sessions=[AccountCacheSessionDTO(session_id=uuid.uuid4(), device="testclient")],
    )

    await storage.save("sessions", str(session_value.user.id), session_value)

    value_from_cache = await storage.get("sessions", str(session_value.user.id), model_type=AccountCacheDTO)
    assert value_from_cache == session_value



from email.message import EmailMessage
import smtplib

from loguru import logger

from .settings import MailSettings, settings


class MailClient:
    def __init__(self, config: MailSettings, debug: bool = False):
        self.settings = config
        self.debug = debug

    def send(self, recipients: list[str], subject: str, body: str):
        if self.debug:
            logger.info(
                "Emulate sending email to {} with subject '{}' and body '{}' in debug mode",
                recipients,
                subject,
                body,
            )
            return

        msg = EmailMessage()
        msg.set_content(body)
        server = smtplib.SMTP(self.settings.host, self.settings.port, timeout=5)
        server.starttls()
        server.login(self.settings.login, self.settings.password)

        msg = EmailMessage()
        msg["Subject"] = subject
        msg.set_content(body)
        msg["From"] = self.settings.login

        for recipient in recipients:
            msg["To"] = recipient
            server.send_message(msg)

        server.quit()


async def provide_mail_client():
    return MailClient(settings.mail_config, settings.debug)



from typing import AsyncGenerator

import pytest


from auth.services import (
    AuthService,
    PermissionService,
    provide_auth_service,
    provide_permission_service,
    provide_role_service,
    provide_user_service,
)
from core.storage import Storage
from tests.integrations import TestMailClient
from tests.factories import PermissionFactory, RoleFactory, UserFactory


@pytest.fixture
async def user_service(db_session):
    return await provide_user_service(db_session)


@pytest.fixture
async def role_service(db_session):
    return await provide_role_service(db_session)


@pytest.fixture
async def auth_service(
    db_session,
    mail_client: TestMailClient,
    user_service,
    application,
    storage: Storage,
) -> AsyncGenerator[AuthService, None]:
    async for auth_service in provide_auth_service(
        db_session=db_session,
        mail_client=mail_client,
        storage=storage,
        user_service=user_service,
    ):
        yield auth_service
    mail_client.reset()
    await storage.disconnect()


@pytest.fixture
async def permission_service(db_session) -> PermissionService:
    return await provide_permission_service(db_session)


@pytest.fixture
async def users_fixture(user_service):
    users = await user_service.create_many([UserFactory.build() for _ in range(10)], auto_commit=True)

    yield users
    await user_service.delete_many([user.id for user in users])


@pytest.fixture
async def roles_fixture(role_service):
    roles = await role_service.create_many(
        [RoleFactory.build() for _ in range(10)],
        auto_commit=True,
    )
    yield roles
    await role_service.delete_many([role.id for role in roles])


@pytest.fixture
async def permissions_fixture(permission_service):
    permissions = await permission_service.create_many([PermissionFactory.build() for _ in range(10)], auto_commit=True)
    yield permissions
    await permission_service.delete_many([permission.id for permission in permissions])



from advanced_alchemy.base import UUIDAuditBase
from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, mapped_column


class RolePermissionModel(UUIDAuditBase):
    __tablename__ = "roles_permissions"

    role_id: Mapped[int] = mapped_column(
        ForeignKey("roles.id"),
        primary_key=True,
    )
    permission_id: Mapped[int] = mapped_column(ForeignKey("permissions.id", ondelete="CASCADE"), primary_key=True)
